package net.knarcraft.placeholdersigns;

import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.knarlib.formatting.Translator;
import net.knarcraft.knarlib.property.ColorConversion;
import net.knarcraft.placeholdersigns.command.CopySignCommand;
import net.knarcraft.placeholdersigns.command.CopySignTextCommand;
import net.knarcraft.placeholdersigns.command.EditSignCommand;
import net.knarcraft.placeholdersigns.command.SetPlaceholderUpdateDelayCommand;
import net.knarcraft.placeholdersigns.command.UnWaxSignCommand;
import net.knarcraft.placeholdersigns.command.ViewSignCommand;
import net.knarcraft.placeholdersigns.config.PlaceholderSignMessage;
import net.knarcraft.placeholdersigns.handler.PlaceholderSignHandler;
import net.knarcraft.placeholdersigns.handler.PlaceholderSignRequestHandler;
import net.knarcraft.placeholdersigns.handler.PlaceholderSignUpdateQueueHandler;
import net.knarcraft.placeholdersigns.listener.ChunkListener;
import net.knarcraft.placeholdersigns.listener.SignBreakListener;
import net.knarcraft.placeholdersigns.listener.SignClickListener;
import net.knarcraft.placeholdersigns.listener.SignTextListener;
import net.knarcraft.placeholdersigns.runnable.SignUpdate;
import org.bukkit.Bukkit;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.PluginCommand;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.jetbrains.annotations.NotNull;

import java.util.logging.Level;

/**
 * This plugin's main class
 */
public final class PlaceholderSigns extends JavaPlugin {

    private static PlaceholderSigns instance;
    private PlaceholderSignHandler signHandler;
    private PlaceholderSignRequestHandler requestHandler;
    private PlaceholderSignUpdateQueueHandler updateQueueHandler;
    private StringFormatter stringFormatter;
    private int signUpdateDelay;

    /**
     * Gets an instance of this plugin
     *
     * @return <p>A plugin instance</p>
     */
    @NotNull
    public static PlaceholderSigns getInstance() {
        return instance;
    }

    /**
     * Gets this instance's placeholder sign handler
     *
     * @return <p>The sign handler</p>
     */
    @NotNull
    public PlaceholderSignHandler getSignHandler() {
        return this.signHandler;
    }

    /**
     * Gets this instance's placeholder sign request handler
     *
     * @return <p>The request handler</p>
     */
    @NotNull
    public PlaceholderSignRequestHandler getRequestHandler() {
        return this.requestHandler;
    }

    /**
     * Gets this instance's update queue handler
     *
     * @return <p>The update queue handler</p>
     */
    @NotNull
    public PlaceholderSignUpdateQueueHandler getUpdateQueueHandler() {
        return this.updateQueueHandler;
    }

    /**
     * Gets the string formatter to use for formatting messages
     *
     * @return <p>The string formatter</p>
     */
    @NotNull
    public StringFormatter getStringFormatter() {
        return this.stringFormatter;
    }

    /**
     * Gets the default sign update delay
     *
     * @return <p>The sign update delay</p>
     */
    public int getSignUpdateDelay() {
        return this.signUpdateDelay;
    }

    @Override
    public void onEnable() {
        instance = this;
        getConfig().options().copyDefaults(true);
        saveConfig();

        Translator translator = new Translator();
        translator.registerMessageCategory(PlaceholderSignMessage.SUCCESS_CLICK_SIGN_TO_EDIT);
        translator.setColorConversion(ColorConversion.RGB);
        translator.loadLanguages(this.getDataFolder(), "en", getConfig().getString("language", "en"));
        this.stringFormatter = new StringFormatter(this.getDescription().getName(), translator);
        this.stringFormatter.setColorConversion(ColorConversion.RGB);
        this.stringFormatter.setNamePrefix("#A5682A[&r&l");
        this.stringFormatter.setNameSuffix("&r#A5682A]");

        signUpdateDelay = getConfig().getInt("defaultSignUpdateTicks", 100);
        if (signUpdateDelay < 1) {
            signUpdateDelay = 100;
        }

        this.signHandler = new PlaceholderSignHandler();
        this.signHandler.load();
        this.requestHandler = new PlaceholderSignRequestHandler();
        this.updateQueueHandler = new PlaceholderSignUpdateQueueHandler();

        if (Bukkit.getPluginManager().getPlugin("PlaceholderAPI") == null) {
            getLogger().log(Level.WARNING, "Could not find PlaceholderAPI! This plugin is required.");
            Bukkit.getPluginManager().disablePlugin(this);
            return;
        }

        Bukkit.getScheduler().runTaskTimer(this, new SignUpdate(this.signHandler), 20, 1);

        PluginManager pluginManager = Bukkit.getPluginManager();
        pluginManager.registerEvents(new SignBreakListener(), this);
        pluginManager.registerEvents(new SignTextListener(), this);
        pluginManager.registerEvents(new SignClickListener(), this);
        pluginManager.registerEvents(new ChunkListener(), this);

        registerCommand("setSignLine", new EditSignCommand());
        registerCommand("viewSign", new ViewSignCommand());
        registerCommand("copySign", new CopySignCommand());
        registerCommand("unWaxSign", new UnWaxSignCommand());
        registerCommand("copySignText", new CopySignTextCommand());
        registerCommand("setPlaceholderUpdateDelay", new SetPlaceholderUpdateDelayCommand());
    }

    @Override
    public void onDisable() {
        // Plugin shutdown logic
    }

    /**
     * Registers a command executor
     *
     * @param commandName <p>The name of the command</p>
     * @param executor    <p>The command's executor</p>
     */
    private void registerCommand(@NotNull String commandName, @NotNull CommandExecutor executor) {
        PluginCommand command = Bukkit.getPluginCommand(commandName);
        if (command != null) {
            command.setExecutor(executor);
        } else {
            getLogger().log(Level.SEVERE, "Unable to register command " + commandName);
        }
    }

}
