package net.knarcraft.placeholdersigns.command;

import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.knarlib.formatting.StringReplacer;
import net.knarcraft.knarlib.util.ColorHelper;
import net.knarcraft.knarlib.util.TabCompletionHelper;
import net.knarcraft.placeholdersigns.PlaceholderSigns;
import net.knarcraft.placeholdersigns.config.PlaceholderSignMessage;
import net.knarcraft.placeholdersigns.container.PlaceholderSign;
import net.md_5.bungee.api.ChatColor;
import org.bukkit.DyeColor;
import org.bukkit.Location;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.block.sign.Side;
import org.bukkit.block.sign.SignSide;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * A generic sign view command
 */
public class ViewSignCommand implements TabExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender commandSender, @NotNull Command command, @NotNull String label,
                             @NotNull String[] arguments) {
        StringFormatter stringFormatter = PlaceholderSigns.getInstance().getStringFormatter();
        if (!(commandSender instanceof Player player)) {
            stringFormatter.displayErrorMessage(commandSender, PlaceholderSignMessage.ERROR_PLAYER_ONLY);
            return false;
        }

        boolean showRawText = true;
        boolean showPlaceholders = true;
        if (arguments.length > 0) {
            showRawText = Boolean.parseBoolean(arguments[0]);
        }
        if (arguments.length > 1) {
            showPlaceholders = Boolean.parseBoolean(arguments[1]);
        }

        Block targetBlock = player.getTargetBlockExact(7);
        if (targetBlock == null || !(targetBlock.getState() instanceof Sign sign)) {
            stringFormatter.displayErrorMessage(commandSender, PlaceholderSignMessage.ERROR_NOT_LOOKING_AT_SIGN);
            return false;
        }

        printSign(sign, player, showRawText, showPlaceholders);
        return true;
    }

    @Nullable
    @Override
    public List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                                      @NotNull String[] arguments) {
        if (arguments.length == 1) {
            return getBooleanTabCompletions(arguments[0]);
        } else if (arguments.length == 2) {
            return getBooleanTabCompletions(arguments[1]);
        } else {
            return new ArrayList<>();
        }
    }

    /**
     * Gets boolean tab-completions
     *
     * @param filter <p>The input argument to filter by</p>
     * @return <p>The the resulting boolean tab-completions</p>
     */
    @NotNull
    private List<String> getBooleanTabCompletions(@NotNull String filter) {
        return TabCompletionHelper.filterMatchingStartsWith(List.of("true", "false"), filter);
    }

    /**
     * Prints the current contents of a sign to a player
     *
     * @param sign   <p>The sign to print</p>
     * @param player <p>The player to display the contents to</p>
     */
    private void printSign(@NotNull Sign sign, @NotNull Player player, boolean showRawText, boolean showPlaceholders) {
        Location location = sign.getLocation();
        SignSide front = sign.getSide(Side.FRONT);
        SignSide back = sign.getSide(Side.BACK);
        String frontLines = showPlaceholders ? getPlaceholderSignText(location, front, Side.FRONT, showRawText) :
                getSignText(front.getLines(), showRawText);
        String backLines = showPlaceholders ? getPlaceholderSignText(location, back, Side.BACK, showRawText) :
                getSignText(back.getLines(), showRawText);

        StringFormatter stringFormatter = PlaceholderSigns.getInstance().getStringFormatter();
        StringReplacer replacer = new StringReplacer(stringFormatter.getUnformattedColoredMessage(
                PlaceholderSignMessage.SUCCESS_SIGN_CONTENTS));
        replacer.add("{frontLines}", frontLines);
        replacer.add("{backLines}", backLines);
        replacer.add("{frontDye}", getDye(front));
        replacer.add("{frontGlow}", getStatus(front.isGlowingText()));
        replacer.add("{backDye}", getDye(back));
        replacer.add("{backGlow}", getStatus(back.isGlowingText()));
        replacer.add("{waxed}", getStatus(sign.isWaxed()));
        player.sendMessage(replacer.replace());
    }

    /**
     * Gets a description of the status of a sign property
     *
     * @param isTrue <p>Whether the property is true</p>
     * @return <p>A description of the property's current status</p>
     */
    @NotNull
    private String getStatus(boolean isTrue) {
        StringFormatter stringFormatter = PlaceholderSigns.getInstance().getStringFormatter();
        if (isTrue) {
            return stringFormatter.getUnformattedColoredMessage(PlaceholderSignMessage.SIGN_PROPERTY_CONFIRM);
        } else {
            return stringFormatter.getUnformattedColoredMessage(PlaceholderSignMessage.SIGN_PROPERTY_DENY);
        }
    }

    /**
     * Gets a description of a dye applied to a sign
     *
     * @param signSide <p>The sign side to get the dye of</p>
     * @return <p>The description of the applied dye</p>
     */
    @NotNull
    private String getDye(@NotNull SignSide signSide) {
        DyeColor dyeColor = signSide.getColor();
        if (dyeColor == null) {
            return "None";
        }

        ChatColor color = ColorHelper.fromColor(dyeColor.getColor());
        return color + signSide.getColor().name();
    }

    /**
     * Gets text from a sign side, and appends it to the given string builder
     *
     * @param signLocation <p>The location of the sign</p>
     * @param signSide     <p>The sign side to get text from</p>
     * @param side         <p>The side of the sign to get placeholders from</p>
     * @param raw          <p>Whether to get the raw text with used formatting codes</p>
     */
    @NotNull
    private String getPlaceholderSignText(@NotNull Location signLocation, @NotNull SignSide signSide,
                                          @NotNull Side side, boolean raw) {
        String[] lines = signSide.getLines();
        PlaceholderSign placeholderSign = PlaceholderSigns.getInstance().getSignHandler().getFromLocation(signLocation);
        if (placeholderSign == null) {
            return getSignText(lines, raw);
        }

        Map<Integer, String> placeholders = placeholderSign.getPlaceholders().get(side);
        if (placeholders != null) {
            for (Map.Entry<Integer, String> entry : placeholders.entrySet()) {
                lines[entry.getKey()] = entry.getValue();
            }
        }

        return getSignText(lines, raw);
    }

    /**
     * Gets text from a sign side, and appends it to the given string builder
     *
     * @param lines <p>The lines on the sign</p>
     * @param raw   <p>Whether to get the raw text with used formatting codes</p>
     */
    @NotNull
    private String getSignText(@NotNull String[] lines, boolean raw) {
        StringBuilder output = new StringBuilder();
        for (int i = 0; i < 4; i++) {
            output.append(i + 1).append(". ");

            String line = lines[i];
            if (raw) {
                output.append(line.replace(ChatColor.COLOR_CHAR, '&'));
            } else {
                output.append(line);
            }
            output.append(ChatColor.COLOR_CHAR).append("r\n");
        }
        return output.toString();
    }

}
